@@ -26,8 +26,6 @@
 			"*wireframe:    False            \n" \
 
 # define release_boing 0
-#undef countof
-#define countof(x) (sizeof((x))/sizeof((*x)))
 
 #include "xlockmore.h"
 #include "gltrackball.h"
