@@ -47,8 +47,8 @@ static const char sccsid[] = "@(#)flurry.c	4.07 97/11/24 xlockmore";
 # define DEFAULTS		"*delay:      10000 \n" \
 						"*showFPS:    False \n"
 
-# define refresh_flurry 0
-# define flurry_handle_event 0
+# define release_flurry 0
+# define flurry_handle_event xlockmore_no_events
 # include "xlockmore.h"		/* from the xscreensaver distribution */
 
 #ifdef USE_GL
@@ -72,10 +72,10 @@ ModStruct   flurry_description = {
     "flurry",
     "init_flurry",
     "draw_flurry",
-    "release_flurry",
+    NULL,
     "draw_flurry",
     "init_flurry",
-    NULL,
+    "free_flurry",
     &flurry_opts,
     1000, 1, 2, 1, 4, 1.0,
     "",
@@ -90,7 +90,6 @@ ModStruct   flurry_description = {
 
 global_info_t *flurry_info = NULL;
 
-static double gTimeCounter = 0.0;
 
 static
 double currentTime(void) {
@@ -105,14 +104,8 @@ double currentTime(void) {
   return (double)tv.tv_sec + (double)tv.tv_usec / 1000000.0;
 }
 
-void OTSetup (void) {
-    if (gTimeCounter == 0.0) {
-        gTimeCounter = currentTime();
-    }
-}
-
-double TimeInSecondsSinceStart (void) {
-    return currentTime() - gTimeCounter;
+double TimeInSecondsSinceStart (const global_info_t *global) {
+    return currentTime() - global->gTimeCounter;
 }
 
 #if 0
@@ -150,7 +143,8 @@ flurry_info_t *new_flurry_info(global_info_t *global, int streams, ColorModes co
     flurry->flurryRandomSeed = RandFlt(0.0, 300.0);
 
 	flurry->fOldTime = 0;
-	flurry->fTime = TimeInSecondsSinceStart() + flurry->flurryRandomSeed;
+	flurry->dframe = 0;
+	flurry->fTime = TimeInSecondsSinceStart(global) + flurry->flurryRandomSeed;
  	flurry->fDeltaTime = flurry->fTime - flurry->fOldTime;
 
     flurry->numStreams = streams;
@@ -231,7 +225,7 @@ void GLRenderScene(global_info_t *global, flurry_info_t *flurry, double b)
     flurry->dframe++;
 
     flurry->fOldTime = flurry->fTime;
-    flurry->fTime = TimeInSecondsSinceStart() + flurry->flurryRandomSeed;
+    flurry->fTime = TimeInSecondsSinceStart(global) + flurry->flurryRandomSeed;
     flurry->fDeltaTime = flurry->fTime - flurry->fOldTime;
 
     flurry->drag = (float) pow(0.9965,flurry->fDeltaTime*85.0);
@@ -320,7 +314,7 @@ ENTRYPOINT void reshape_flurry(ModeInfo *mi, int width, int height)
 {
     global_info_t *global = flurry_info + MI_SCREEN(mi);
 
-    glXMakeCurrent(MI_DISPLAY(mi), global->window, *(global->glx_context));
+    glXMakeCurrent(MI_DISPLAY(mi), global->window, *global->glx_context);
 
     glViewport(0.0, 0.0, width, height);
     glMatrixMode(GL_PROJECTION);
@@ -349,15 +343,12 @@ init_flurry(ModeInfo * mi)
         PRESET_MAX
     } preset_num;
 
-    if (flurry_info == NULL) {
-	OTSetup();
-	if ((flurry_info = (global_info_t *) calloc(MI_NUM_SCREENS(mi),
-			sizeof (global_info_t))) == NULL)
-	    return;
-    }
+    MI_INIT (mi, flurry_info);
 
     global = &flurry_info[screen];
 
+    global->gTimeCounter = currentTime();
+
     global->window = MI_WINDOW(mi);
 
     global->flurry = NULL;
@@ -468,13 +459,14 @@ init_flurry(ModeInfo * mi)
     } else {
 	MI_CLEARWINDOW(mi);
     }
+
+    global->first = 1;
+    global->oldFrameTime = -1;
 }
 
 ENTRYPOINT void
 draw_flurry(ModeInfo * mi)
 {
-    static int first = 1;
-    static double oldFrameTime = -1;
     double newFrameTime;
     double deltaFrameTime = 0;
     double brite;
@@ -486,7 +478,7 @@ draw_flurry(ModeInfo * mi)
     Window      window = MI_WINDOW(mi);
 
     newFrameTime = currentTime();
-    if (oldFrameTime == -1) {
+    if (global->oldFrameTime == -1) {
 	/* special case the first frame -- clear to black */
 	alpha = 1.0;
     } else {
@@ -499,27 +491,27 @@ draw_flurry(ModeInfo * mi)
 	 * than that and the blending causes the display to
 	 * saturate, which looks really ugly.
 	 */
-	if (newFrameTime - oldFrameTime < 1/60.0) {
-	    usleep(MAX_(1,(int)(20000 * (newFrameTime - oldFrameTime))));
+	if (newFrameTime - global->oldFrameTime < 1/60.0) {
+	    usleep(MAX_(1,(int)(20000 * (newFrameTime - global->oldFrameTime))));
 	    return;
 
 	}
-	deltaFrameTime = newFrameTime - oldFrameTime;
+	deltaFrameTime = newFrameTime - global->oldFrameTime;
 	alpha = 5.0 * deltaFrameTime;
     }
-    oldFrameTime = newFrameTime;
+    global->oldFrameTime = newFrameTime;
 
     if (alpha > 0.2) alpha = 0.2;
 
     if (!global->glx_context)
 	return;
 
-    if (first) {
-	MakeTexture();
-	first = 0;
+    if (global->first) {
+	global->texid = MakeTexture();
+	global->first = 0;
     }
     glDrawBuffer(GL_BACK);
-    glXMakeCurrent(display, window, *(global->glx_context));
+    glXMakeCurrent(display, window, *global->glx_context);
 
     glEnable(GL_BLEND);
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
@@ -539,27 +531,17 @@ draw_flurry(ModeInfo * mi)
 }
 
 ENTRYPOINT void
-release_flurry(ModeInfo * mi)
+free_flurry(ModeInfo * mi)
 {
-    if (flurry_info != NULL) {
-	int screen;
+    global_info_t *global = &flurry_info[MI_SCREEN(mi)];
+    flurry_info_t *flurry;
 
-	for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-	    global_info_t *global = &flurry_info[screen];
-	    flurry_info_t *flurry;
+	if (!global->glx_context) return;
+    glXMakeCurrent(MI_DISPLAY(mi), global->window, *global->glx_context);
 
-	    if (global->glx_context) {
-		glXMakeCurrent(MI_DISPLAY(mi), global->window, *(global->glx_context));
-	    }
-
-	    for (flurry = global->flurry; flurry; flurry=flurry->next) {
+    for (flurry = global->flurry; flurry; flurry=flurry->next)
 		delete_flurry_info(flurry);
-	    }
-	}
-	(void) free((void *) flurry_info);
-	flurry_info = NULL;
-    }
-    FreeAllGL(mi);
+	if (global->texid) glDeleteTextures (1, &global->texid);
 }
 
 XSCREENSAVER_MODULE ("Flurry", flurry)
