@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 1992-2020 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright © 1992-2022 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -83,6 +83,40 @@
         of your screen saver module.  See .../hacks/config/README for details.
  */
 
+
+/* Flow of control for the "screenhack" API:
+
+      main
+      xscreensaver_function_table->setup_cb => none
+      XtAppInitialize
+      pick_visual
+      init_window
+      run_screenhack_table (loops forever)
+      !  xscreensaver_function_table.init_cb => HACK_init (once)
+         usleep_and_process_events
+      !    xscreensaver_function_table.event_cb => HACK_event
+      !  xscreensaver_function_table.draw_cb => HACK_draw
+
+   The "xlockmore" API (which is also the OpenGL API) acts like a screenhack
+   but then adds another layer underneath that, before eventually running the
+   hack's methods.  Flow of control for the "xlockmore" API:
+
+      main
+      + xscreensaver_function_table->setup_cb => xlockmore_setup (opt handling)
+      XtAppInitialize
+      pick_visual
+      init_window
+      run_screenhack_table (loops forever)
+      !  xscreensaver_function_table.init_cb => xlockmore_init (once)
+         usleep_and_process_events
+      !    xscreensaver_function_table.event_cb => xlockmore_event
+      +      xlockmore_function_table.hack_handle_events => HACK_handle_event
+      !  xscreensaver_function_table.draw_cb => xlockmore_draw
+      +    xlockmore_function_table.hack_init => init_HACK
+      +      init_GL (eglCreateContext or glXCreateContext)
+      +    xlockmore_function_table.hack_draw => draw_HACK
+ */
+
 #define DEBUG_PAIR
 
 #include <stdio.h>
@@ -97,17 +131,8 @@
 # include <X11/SGIScheme.h>	/* for SgiUseSchemes() */
 #endif /* __sgi */
 
-#ifdef HAVE_XMU
-# ifndef VMS
-#  include <X11/Xmu/Error.h>
-# else /* VMS */
-#  include <Xmu/Error.h>
-# endif
-#else
-# include "xmu.h"
-#endif
-
 #include "screenhackI.h"
+#include "xmu.h"
 #include "version.h"
 #include "vroot.h"
 #include "fps.h"
@@ -338,8 +363,8 @@ pick_visual (Screen *screen)
 
 
 /* Notice when the user has requested a different visual or colormap
-   on a pre-existing window (e.g., "-root -visual truecolor" or
-   "-window-id 0x2c00001 -install") and complain, since when drawing
+   on a pre-existing window (e.g., "--root --visual truecolor" or
+   "--window-id 0x2c00001 --install") and complain, since when drawing
    on an existing window, we have no choice about these things.
  */
 static void
@@ -360,9 +385,9 @@ visual_warning (Screen *screen, Window window, Visual *visual, Colormap cmap,
     sprintf (win, "window 0x%lx", (unsigned long) window);
 
   if (window_p)
-    sprintf (why, "-window-id 0x%lx", (unsigned long) window);
+    sprintf (why, "--window-id 0x%lx", (unsigned long) window);
   else
-    strcpy (why, "-root");
+    strcpy (why, "--root");
 
   if (visual_string && *visual_string)
     {
@@ -444,10 +469,13 @@ screenhack_table_handle_events (Display *dpy,
                                 )
 {
   XtAppContext app = XtDisplayToApplicationContext (dpy);
+  XtInputMask m = XtAppPending (app);
 
-  if (XtAppPending (app) & (XtIMTimer|XtIMAlternateInput))
-    XtAppProcessEvent (app, XtIMTimer|XtIMAlternateInput);
+  /* Process non-X11 Xt events (timers, files, signals) without blocking. */
+  if (m & ~XtIMXEvent)
+    XtAppProcessEvent (app, ~XtIMXEvent);
 
+  /* Process all pending X11 events without blocking. */
   while (XPending (dpy))
     {
       XEvent event;
@@ -475,8 +503,10 @@ screenhack_table_handle_events (Display *dpy,
         if (! screenhack_handle_event_1 (dpy, &event))
           return False;
 
-      if (XtAppPending (app) & (XtIMTimer|XtIMAlternateInput))
-        XtAppProcessEvent (app, XtIMTimer|XtIMAlternateInput);
+      /* Last chance to process Xt timers before blocking. */
+      m = XtAppPending (app);
+      if (m & ~XtIMXEvent)
+        XtAppProcessEvent (app, ~XtIMXEvent);
     }
 
 # ifdef EXIT_AFTER
@@ -611,12 +641,14 @@ run_screenhack_table (Display *dpy,
   if (anim_state) screenhack_record_anim_free (anim_state);
 #endif
 
-  ft->free_cb (dpy, window, closure);
   if (fpst) ft->fps_free (fpst);
+#ifdef DEBUG_PAIR
+  if (fpst2) ft->fps_free (fpst2);
+#endif
 
+  ft->free_cb (dpy, window, closure);
 #ifdef DEBUG_PAIR
   if (window2) ft->free_cb (dpy, window2, closure2);
-  if (fpst2) ft->fps_free (fpst2);
 #endif
 }
 
@@ -753,6 +785,15 @@ main (int argc, char **argv)
   SgiUseSchemes ("none"); 
 #endif /* __sgi */
 
+  /* Xt and xscreensaver predate the "--arg" convention, so convert
+     double dashes to single. */
+  {
+    int i;
+    for (i = 1; i < argc; i++)
+      if (argv[i][0] == '-' && argv[i][1] == '-')
+        argv[i]++;
+  }
+
   toplevel = XtAppInitialize (&app, progclass, merged_options,
 			      merged_options_size, &argc, argv,
 			      merged_defaults, 0, 0);
@@ -792,15 +833,13 @@ main (int argc, char **argv)
 
   if (argc > 1)
     {
-      const char *s;
       int i;
       int x = 18;
       int end = 78;
       Bool help_p = (!strcmp(argv[1], "-help") ||
                      !strcmp(argv[1], "--help"));
       fprintf (stderr, "%s\n", version);
-      for (s = progclass; *s; s++) fprintf(stderr, " ");
-      fprintf (stderr, "  https://www.jwz.org/xscreensaver/\n\n");
+      fprintf (stderr, "\n\thttps://www.jwz.org/xscreensaver/\n\n");
 
       if (!help_p)
 	fprintf(stderr, "Unrecognised option: %s\n", argv[1]);
@@ -816,7 +855,7 @@ main (int argc, char **argv)
 	      x = 18;
 	    }
 	  x += size;
-	  fprintf (stderr, "%s", sw);
+	  fprintf (stderr, "-%s", sw);  /* two dashes */
 	  if (argp) fprintf (stderr, " <arg>");
 	  if (i != merged_options_size - 1) fprintf (stderr, ", ");
 	}
