@@ -34,6 +34,7 @@ struct state {
   Screen *screen;
 
   int sizex, sizey; /* screen size */
+  int depth;
   int delay;
   int duration;
   time_t start_time;
@@ -105,6 +106,7 @@ spotlight_init (Display *dpy, Window window)
   st->screen = xgwa.screen;
   st->sizex = xgwa.width;
   st->sizey = xgwa.height;
+  st->depth = xgwa.depth;
   cmap = xgwa.colormap;
   bg = get_pixel_resource (st->dpy, cmap, "background", "Background");
 
@@ -116,7 +118,8 @@ spotlight_init (Display *dpy, Window window)
   st->radius = get_integer_resource (st->dpy, "radius", "Integer");
   if (st->radius < 0) st->radius = 125;
 
-  if (xgwa.width > 2560) st->radius *= 2;  /* Retina displays */
+  if (xgwa.width > 2560 || xgwa.height > 2560)
+    st->radius *= 2;  /* Retina displays */
 
   /* Don't let the spotlight be bigger than the window */
   while (st->radius > xgwa.width * 0.45)
@@ -129,17 +132,11 @@ spotlight_init (Display *dpy, Window window)
 
   /* do the dance */
   gcv.function = GXcopy;
-  gcv.subwindow_mode = IncludeInferiors;
   gcflags = GCForeground | GCFunction;
   gcv.foreground = bg;
-
-#ifdef NOPE
-  if (use_subwindow_mode_p(xgwa.screen, st->window)) /* see grabscreen.c */
-    gcflags |= GCSubwindowMode;
-#endif
   st->window_gc = XCreateGC(st->dpy, st->window, gcflags, &gcv);
 
-  st->pm = XCreatePixmap(st->dpy, st->window, st->sizex, st->sizey, xgwa.depth);
+  st->pm = XCreatePixmap(st->dpy, st->window, st->sizex, st->sizey, st->depth);
   XClearWindow(st->dpy, st->window);
 
   st->first_time = 1;
@@ -148,12 +145,15 @@ spotlight_init (Display *dpy, Window window)
 #ifdef HAVE_JWXYZ	/* Don't second-guess Quartz's double-buffering */
   st->buffer = 0;
 #else
-  st->buffer = XCreatePixmap(st->dpy, st->window, st->sizex, st->sizey, xgwa.depth);
+  st->buffer = XCreatePixmap(st->dpy, st->window, st->sizex, st->sizey,
+                             st->depth);
 #endif
 
-  st->buffer_gc = XCreateGC(st->dpy, (st->buffer ? st->buffer : window), gcflags, &gcv);
+  st->buffer_gc = XCreateGC(st->dpy, (st->buffer ? st->buffer : window),
+                            gcflags, &gcv);
   if (st->buffer)
-    XFillRectangle(st->dpy, st->buffer, st->buffer_gc, 0, 0, st->sizex, st->sizey);
+    XFillRectangle(st->dpy, st->buffer, st->buffer_gc, 0, 0, st->sizex,
+                   st->sizey);
 
   /* create clip mask (so it's a circle, not a square) */
   clip_pm = XCreatePixmap(st->dpy, st->window, st->radius*4, st->radius*4, 1);
@@ -192,7 +192,7 @@ spotlight_init (Display *dpy, Window window)
  * perform one iteration
  */
 static void
-onestep (struct state *st, Bool first_p)
+onestep (struct state *st)
 {
   long now;
   unsigned long now_unsigned;
@@ -216,9 +216,6 @@ onestep (struct state *st, Bool first_p)
 
 #define nrnd(x) (random() % (x))
 
-  st->oldx = st->x;
-  st->oldy = st->y;
-
   st->s = st->radius *4 ;   /* s = width of buffer */
 
   now_unsigned = (unsigned long) currentTimeInMs(st) + st->off;
@@ -232,6 +229,9 @@ onestep (struct state *st, Bool first_p)
     ((1 + sin(((double)now) / Y_PERIOD * 2. * M_PI))/2.0) 
     * (st->geom.height - st->s/2) -st->s/4;
     
+  st->oldx = st->x;
+  st->oldy = st->y;
+
   if (!st->first_p)
     {
       /* limit change in x and y to buffer width */
@@ -240,6 +240,10 @@ onestep (struct state *st, Bool first_p)
       if ( st->y < (st->oldy - st->max_y_speed) ) st->y = st->oldy - st->max_y_speed;
       if ( st->y > (st->oldy + st->max_y_speed) ) st->y = st->oldy + st->max_y_speed;
     }
+  else
+    {
+      st->first_p = False;
+    }
 
   if (! st->buffer)
     {
@@ -286,15 +290,17 @@ static unsigned long
 spotlight_draw (Display *dpy, Window window, void *closure)
 {
   struct state *st = (struct state *) closure;
-  onestep(st, st->first_p);
-  st->first_p = False;
+  onestep(st);
   return st->delay;
 }
   
 static void
 spotlight_reshape (Display *dpy, Window window, void *closure, 
-                 unsigned int w, unsigned int h)
+                   unsigned int w, unsigned int h)
 {
+  struct state *st = (struct state *) closure;
+  XClearWindow(st->dpy, st->window);
+  /* Should resize the playfield, but let's at least not show whitespace */
 }
 
 static Bool
@@ -328,11 +334,6 @@ static const char *spotlight_defaults [] = {
   ".foreground:			white",
   "*dontClearRoot:		True",
   "*fpsSolid:			true",
-
-#ifdef __sgi	/* really, HAVE_READ_DISPLAY_EXTENSION */
-  "*visualID:			Best",
-#endif
-
   "*delay:			10000",
   "*duration:			120",
   "*radius:			125",
