@@ -55,7 +55,8 @@ static const char sccsid[] = "@(#)strange.c	5.00 2000/11/01 xlockmore";
 					"*ignoreRotation: True \n" \
 					"*useSHM: True \n" \
 					"*useThreads: True \n" \
-				    "*lowrez: True \n" \
+
+/*				    "*lowrez: True \n" \ */
 
 # define SMOOTH_COLORS
 # define release_strange 0
@@ -1084,7 +1085,12 @@ init_strange(ModeInfo * mi)
 	GC          gc = MI_GC(mi);
 #endif
 	ATTRACTOR  *Attractor;
-	size_t      pointStructSize =
+	size_t      pointStructSize;
+
+    if (MI_WIDTH(mi) > 2560 || MI_HEIGHT(mi) > 2560)
+      pointSize *= 3;  /* Retina displays */
+
+    pointStructSize =
 		pointSize == 1 ? sizeof (XPoint) : sizeof (XRectangle);
 
 	if (curve <= 0) curve = 10;
