@@ -20,9 +20,9 @@
  * so you see a ghostly, misaligned image.
  */
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif /* HAVE_CONFIG_H */
+#include "screenhack.h"
+#include "ximage-loader.h"
+#include "analogtv.h"
 
 #include <math.h>
 
@@ -34,10 +34,6 @@
 # include <X11/Intrinsic.h> /* for XtDatabase in hack_resources() */
 #endif
 
-#include "screenhack.h"
-#include "ximage-loader.h"
-#include "analogtv.h"
-
 #define USE_TEST_PATTERNS
 
 #include "images/gen/logo-180_png.h"
