@@ -11,6 +11,7 @@
  */
 
 #include <math.h>
+#include <limits.h>
 #include "screenhack.h"
 
 #ifndef MIN
@@ -24,8 +25,8 @@
 #define MINX 0.0
 #define MINY 0.0
 /* This should be *way* slower than the spotlight hack was */
-#define X_PERIOD 45000.0
-#define Y_PERIOD 36000.0
+#define X_PERIOD (45000.0 * 3)
+#define Y_PERIOD (36000.0 * 3)
 
 struct state {
   Display *dpy;
@@ -56,13 +57,15 @@ struct state {
 static long currentTimeInMs(struct state *st)
 { 
   struct timeval curTime;
+  unsigned long ret_unsigned;
 #ifdef GETTIMEOFDAY_TWO_ARGS
   struct timezone tz = {0,0};
   gettimeofday(&curTime, &tz);
 #else
   gettimeofday(&curTime);
 #endif
-  return curTime.tv_sec*1000 + curTime.tv_usec/1000.0;
+  ret_unsigned = curTime.tv_sec *1000U + curTime.tv_usec / 1000;
+  return (ret_unsigned <= LONG_MAX) ? ret_unsigned : -1 - (long)(ULONG_MAX - ret_unsigned);
 }
 
 static void *
@@ -103,18 +106,30 @@ zoom_init (Display *dpy, Window window)
   st->pixspacey = get_integer_resource(st->dpy, "pixspacey", "Integer");
   if (st->pixspacey < 0)
     st->pixspacey = 0;
+
+  if (st->sizex < 50 || st->sizey < 50) {  /* tiny window */
+    st->pixwidth = 10;
+    st->pixheight = 10;
+  }
+
   st->lenses = get_boolean_resource(st->dpy, "lenses", "Boolean");
   st->lensoffsetx = get_integer_resource(st->dpy, "lensoffsetx", "Integer");
   st->lensoffsetx = MAX(0, MIN(st->pixwidth, st->lensoffsetx));
   st->lensoffsety = get_integer_resource(st->dpy, "lensoffsety", "Integer");
   st->lensoffsety = MAX(0, MIN(st->pixwidth, st->lensoffsety));
 
+  if (st->sizex > 2560 || st->sizey > 2560)  /* Retina displays */
+    {
+      int s = 3;
+      st->pixwidth  *= s;
+      st->pixheight *= s;
+      st->pixspacex *= s;
+      st->pixspacey *= s;
+    }
+
   gcv.function = GXcopy;
-  gcv.subwindow_mode = IncludeInferiors;
   gcflags = GCForeground|GCFunction;
   gcv.foreground = bg;
-  if (!st->lenses && use_subwindow_mode_p(xgwa.screen, st->window))       /* see grabscreen.c */
-    gcflags |= GCSubwindowMode;
   st->window_gc = XCreateGC(st->dpy, st->window, gcflags, &gcv);
 
 
@@ -124,15 +139,10 @@ zoom_init (Display *dpy, Window window)
   XFillRectangle(st->dpy, st->window, st->window_gc, 0, 0, st->sizex, st->sizey);
   XSetWindowBackground(st->dpy, st->window, bg);
 
-  st->start_time = time ((time_t) 0);
+  st->start_time = time ((time_t *) 0);
   st->img_loader = load_image_async_simple (0, xgwa.screen, st->window,
                                             st->pm, 0, 0);
 
-  /* We might have needed this to grab the image, but if we leave this set
-     to GCSubwindowMode, then we'll *draw* right over subwindows too. */
-  XSetSubwindowMode (st->dpy, st->window_gc, ClipByChildren);
-
-
   nblocksx = (int)ceil((double)st->sizex / (double)(st->pixwidth + st->pixspacex));
   nblocksy = (int)ceil((double)st->sizey / (double)(st->pixheight + st->pixspacey));
   if (st->lenses)
@@ -153,13 +163,14 @@ zoom_draw (Display *dpy, Window window, void *closure)
   unsigned x, y, i, j;
 
   long now;
+  unsigned long now_unsigned;
 
   if (st->img_loader)   /* still loading */
     {
       st->img_loader = load_image_async_simple (st->img_loader, 0, 0, 0, 0, 0);
       if (! st->img_loader) {  /* just finished */
         XClearWindow (st->dpy, st->window);
-        st->start_time = time ((time_t) 0);
+        st->start_time = time ((time_t *) 0);
         if (!st->lenses) {
           st->orig_map = XGetImage(st->dpy, st->pm, 0, 0, st->sizex, st->sizey, ~0L, ZPixmap);
 /*          XFreePixmap(st->dpy, st->pm);
@@ -170,7 +181,7 @@ zoom_draw (Display *dpy, Window window, void *closure)
     }
 
   if (!st->img_loader &&
-      st->start_time + st->duration < time ((time_t) 0)) {
+      st->start_time + st->duration < time ((time_t *) 0)) {
     st->img_loader = load_image_async_simple (0, st->screen, st->window,
                                               st->pm, 0, 0);
     return st->delay;
@@ -179,7 +190,8 @@ zoom_draw (Display *dpy, Window window, void *closure)
 #define nrnd(x) (random() % (x))
 
   now = currentTimeInMs(st);
-  now += st->sinusoid_offset;  /* don't run multiple screens in lock-step */
+  now_unsigned = (unsigned long) now + st->sinusoid_offset;  /* don't run multiple screens in lock-step */
+  now = (now_unsigned <= LONG_MAX) ? now_unsigned : -1 - (long)(ULONG_MAX - now_unsigned);
 
   /* find new x,y */
   st->tlx = ((1. + sin(((double)now) / X_PERIOD * 2. * M_PI))/2.0)
@@ -218,6 +230,12 @@ zoom_reshape (Display *dpy, Window window, void *closure,
 static Bool
 zoom_event (Display *dpy, Window window, void *closure, XEvent *event)
 {
+  struct state *st = (struct state *) closure;
+  if (screenhack_event_helper (dpy, window, event))
+    {
+      st->start_time = 0;
+      return True;
+    }
   return False;
 }
 
@@ -234,23 +252,22 @@ zoom_free (Display *dpy, Window window, void *closure)
 
 static const char *zoom_defaults[] = {
   "*dontClearRoot: True",
-  ".foreground: white",
-  ".background: #111111",
-  "*fpsSolid:	true",
-#ifdef __sgi /* really, HAVE_READ_DISPLAY_EXTENSION */
-  "*visualID: Best",
-#endif
-  "*lenses:      true",
-  "*delay:       10000",
-  "*duration:    120",
-  "*pixwidth:    10",
-  "*pixheight:   10",
-  "*pixspacex:   2",
-  "*pixspacey:   2",
-  "*lensoffsetx: 5",
-  "*lensoffsety: 5",
-#ifdef USE_IPHONE
+  ".foreground:    white",
+  ".background:    #111111",
+/* ".lowrez:        true", */
+  "*fpsSolid:	   true",
+  "*lenses:        true",
+  "*delay:         10000",
+  "*duration:      120",
+  "*pixwidth:      40",
+  "*pixheight:     40",
+  "*pixspacex:     2",
+  "*pixspacey:     2",
+  "*lensoffsetx:   5",
+  "*lensoffsety:   5",
+#ifdef HAVE_MOBILE
   "*ignoreRotation: True",
+  "*rotateImages:   True",
 #endif
   0
 };
